#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import sys, threading

if '--help' in sys.argv:
    print("""client.py
    --ssl
        Encrypt communication with SSL using M2Crypto. Requires a
        server.pem and server.key in the current directory.
""")
    sys.exit(0)

if '--ssl' in sys.argv:
    # Set up a SSL socket
    import M2Crypto
    ctx = M2Crypto.SSL.Context()
    ctx.load_cert('server.pem', 'server.key')
    s = M2Crypto.SSL.Connection(ctx)
else:
    # Set up a TCP socket
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

#  Start listening on the socket for connections
s.bind(('', 4712))
s.listen(1)

s2 = s.accept()[0]

data = s2.recv(1024)
while data:
    print(repr(data))
    s2.send(data)
    data = s2.recv(1024)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import symmetricjsonrpc3, sys

class PongRPCServer(symmetricjsonrpc3.RPCServer):
    class InboundConnection(symmetricjsonrpc3.RPCServer.InboundConnection):
        class Thread(symmetricjsonrpc3.RPCServer.InboundConnection.Thread):
            class Request(symmetricjsonrpc3.RPCServer.InboundConnection.Thread.Request):
                def dispatch_notification(self, subject):
                    print("dispatch_notification(%s)" % (repr(subject),))
                    assert subject['method'] == "shutdown"
                    # Shutdown the server. Note: We must use a
                    # notification, not a method for this - when the
                    # server's dead, there's no way to inform the
                    # client that it is...
                    self.parent.parent.parent.shutdown()

                def dispatch_request(self, subject):
                    print("dispatch_request(%s)" % (repr(subject),))
                    assert subject['method'] == "ping"
                    # Call the client back
                    # self.parent is a symmetricjsonrpc3.RPCClient subclass (see the client code for more examples)
                    res = self.parent.request("pingping", wait_for_response=True)
                    print("parent.pingping => %s" % (repr(res),))
                    assert res == "pingpong"
                    return "pong"

if '--help' in sys.argv:
    print("""client.py
    --ssl
        Encrypt communication with SSL using M2Crypto. Requires a
        server.pem and server.key in the current directory.
""")
    sys.exit(0)

if '--ssl' in sys.argv:
    # Set up a SSL socket
    import M2Crypto
    ctx = M2Crypto.SSL.Context()
    ctx.load_cert('server.pem', 'server.key')
    s = M2Crypto.SSL.Connection(ctx)
else:
    # Set up a TCP socket
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

#  Start listening on the socket for connections
s.bind(('', 4712))
s.listen(1)

# Create a server thread handling incoming connections
server = PongRPCServer(s, name="PongServer")

# Wait for the server to stop serving clients
server.join()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Utilities for abstracting I/O for sockets or file-like objects
behind an identical interface."""
import io
import select
from logging import getLogger


logger = getLogger(__name__)

debug_write = False
debug_read = False


class WriterWrapper:
    """Provides a unified interface for writing to sockets or
    file-like objects.

    Its instances will actually belong to one of its subclasses,
    depending on what type of object it wraps."""
    poll_timeout = 1000
    buff_maxsize = 512

    def __new__(cls, f):
        if cls is not WriterWrapper:
            return object.__new__(cls)
        elif hasattr(f, "send"):
            return SocketWriter(f)
        elif hasattr(f, "write"):
            return FileWriter(f)
        else:
            return f

    def __init__(self, f):
        self.f = f
        self.buff = None
        self.poll = None
        if _is_real_file(f):
            self.poll = select.poll()
            self.poll.register(f, select.POLLOUT | select.POLLERR | select.POLLHUP | select.POLLNVAL)
        self.closed = False

    @property
    def buff_len(self):
        return len(self.buff) if self.buff else 0

    def close(self):
        self.closed = True
        self.f.close()

    def write(self, s):
        if self.buff is None:
            self.buff = s
        else:
            self.buff += s
        if self.buff_len > self.buff_maxsize:
            self.flush()

    def flush(self):
        data = self.buff
        self.buff = None
        if debug_write:
            logger.debug("write(%s)", repr(data))
        while data:
            self._wait()
            data = data[self._write(data):]

    def _wait(self):
        if not self.poll:
            return
        res = []
        while not res and not self.closed:
            res = self.poll.poll(self.poll_timeout)
        if self.closed:
            raise EOFError

    def _write(self, s):
        raise NotImplementedError


class FileWriter(WriterWrapper):
    def _write(self, s):
        self.f.write(s)
        return len(s)


class SocketWriter(WriterWrapper):
    def _write(self, s):
        res = self.f.send(s.encode('ascii'))
        return res


class ReaderWrapper:
    """Provides a unified interface for reading from sockets or
    file-like objects.

    Its instances will actually belong to one of its subclasses,
    depending on what type of object it wraps."""
    poll_timeout = 1000

    def __new__(cls, f):
        if cls is not ReaderWrapper:
            return object.__new__(cls)
        elif hasattr(f, "recv"):
            return SocketReader(f)
        elif hasattr(f, "read"):
            return FileReader(f)
        else:
            return f

    def __init__(self, f):
        self.file = f
        self.poll = None
        if _is_real_file(f):
            self.poll = select.poll()
            self.poll.register(f, select.POLLIN | select.POLLPRI | select.POLLERR | select.POLLHUP | select.POLLNVAL)
        self.closed = False

    def __iter__(self):
        return self

    def __next__(self):
        try:
            self._wait()
        except EOFError:
            raise StopIteration
        result = self._read()
        if result == '':
            raise StopIteration
        else:
            if debug_read:
                logger.debug("read(%s)", repr(result))
            return result

    def close(self):
        self.closed = True
        self.file.close()

    def _wait(self):
        if not self.poll:
            return
        res = []
        while not res and not self.closed:
            res = self.poll.poll(self.poll_timeout)
        if self.closed:
            raise EOFError

    def _read(self):
        raise NotImplementedError


class FileReader(ReaderWrapper):
    def _read(self):
        return self.file.read(1)


class SocketReader(ReaderWrapper):
    def _read(self):
        return self.file.recv(1).decode('ascii')


class ReIterator:
    """An iterator wrapper that provides lookahead through the peek
    method."""
    def __init__(self, i):
        self._prefix = [] # In reverse order!
        self._i = iter(i)

    def __iter__(self):
        return self

    def next(self):
        if self._prefix:
            return self._prefix.pop(0)
        return next(self._i)

    def _put(self, value):
        self._prefix.append(value)

    def peek(self):
        try:
            if not self._prefix:
                self._put(next(self._i))
            return self._prefix[-1]
        except StopIteration:
            raise EOFError()


def _is_real_file(f):
    if hasattr(f, 'fileno'):
        try:
            return f.fileno() is not None
        except io.UnsupportedOperation:
            pass
    return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import symmetricjsonrpc3, sys

class PingRPCClient(symmetricjsonrpc3.RPCClient):
    class Request(symmetricjsonrpc3.RPCClient.Request):
        def dispatch_request(self, subject):
            # Handle callbacks from the server
            print("dispatch_request(%s)" % (repr(subject),))
            assert subject['method'] == "pingping"
            return "pingpong"

if '--help' in sys.argv:
    print("""client.py
    --ssl
        Encrypt communication with SSL using M2Crypto. Requires a
        server.pem in the current directory.
""")
    sys.exit(0)

if '--ssl' in sys.argv:
    # Set up an SSL connection
    import M2Crypto
    ctx = M2Crypto.SSL.Context()
    ctx.set_verify(M2Crypto.SSL.verify_peer | M2Crypto.SSL.verify_fail_if_no_peer_cert, depth=9)
    if ctx.load_verify_locations('server.pem') != 1: raise Exception('No CA certs')
    s = M2Crypto.SSL.Connection(ctx)
else:
    # Set up a TCP socket
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

#  Connect to the server
s.connect(('localhost', 4712))

# Create a client thread handling for incoming requests
client = PingRPCClient(s)

# Call a method on the server
res = client.request("ping", wait_for_response=True)
print("client.ping => %s" % (repr(res),))
assert res == "pong"

# Notify server it can shut down
client.notify("shutdown")

client.shutdown()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""JSON (de)serialization facilities."""

import io
import sys
from logging import getLogger

from . import wrappers


logger = getLogger(__name__)


def from_json(str):
    """Return a Python object representing the JSON value in str."""
    r = Reader(str)
    return r.read_value()


def to_json(obj):
    """Return a JSON string representing the Python object obj."""
    i = io.StringIO()
    w = Writer(i, encoding='UTF-8')
    w.write_value(obj)
    return i.getvalue()


class Writer:
    """A serializer for Python values to JSON. Allowed types for
    values to serialize are:

        * None
        * True
        * False
        * int
        * float
        * str
        * bytes
        * list
        * dict (keys must be str)
        * any object with a __to_json__ method

    The writer must be instantiated with a file-like object to write
    the serialized JSON to as sole argument. To actually serialize
    data, call the write_value() or write_values() methods."""

    def __init__(self, s, encoding=None):
        self.encoding = encoding
        self.s = wrappers.WriterWrapper(s)

    def close(self):
        self.s.close()

    def write_value(self, value):
        self.unflushed_write_value(value)
        self.s.flush()

    def unflushed_write_value(self, value):
        if hasattr(value, '__to_json__'):
            self.unflushed_write_value(value.__to_json__())
        elif isinstance(value, str):
            self.s.write('"')
            for c in value:
                if c == '\b':
                    self.s.write(r'\b')
                elif c == '\t':
                    self.s.write(r'\t')
                elif c == '\n':
                    self.s.write(r'\n')
                elif c == '\f':
                    self.s.write(r'\f')
                elif c == '\r':
                    self.s.write(r'\r')
                elif c == '"':
                    self.s.write(r'\"')
                elif c == '\\':
                    self.s.write(r'\\')
                elif c >= ' ' and c <= '~':
                    self.s.write(c)
                elif c > '~':
                    self.s.write(r'\u%04x' % ord(c))
                else:
                    raise Exception("Cannot encode character %x into json string" % ord(c))
            self.s.write('"')
        elif isinstance(value, bytes):
            self.unflushed_write_value(value.decode(self.encoding or sys.getdefaultencoding()))
        elif isinstance(value, bool):
            self.s.write(value and 'true' or 'false')
        elif isinstance(value, (int, float)):
            r = repr(value)
            if r[-1] == 'L':
                r = r[:-1]
            self.s.write(r)
        elif value is None:
            self.s.write('null')
        elif hasattr(value, '__iter__'):
            if hasattr(value,'items'):
                self.s.write('{')
                for n, (k, v) in enumerate(value.items()):
                    if (n > 0):
                        self.s.write(',')
                    self.unflushed_write_value(k)
                    self.s.write(':')
                    self.unflushed_write_value(v)
                self.s.write('}')
            else:
                self.s.write('[')
                for n, i in enumerate(value):
                    if (n > 0):
                        self.s.write(',')
                    self.unflushed_write_value(i)
                self.s.write(']')
        else:
            raise Exception("Cannot encode %s of type %s to json" % (value,type(value)))

    def unflushed_write_values(self, values):
        for value in values:
            self.unflushed_write_value(value)


class Tokenizer:
    """A SAX-like recursive-descent parser for JSON.

    This class does not actually parse JSON into Python objects, it
    only provides tokenization (just like a SAX parser for XML).

    This class must be subclassed to be useful. See Reader for
    a full example."""

    def __init__(self, s):
        self.closable = wrappers.ReaderWrapper(s)
        self.s = wrappers.ReIterator(self.closable)

    # Override these in a subclass to actually do something with the
    # parsed data
    def pair_begin(self): pass
    def pair_end(self): pass
    def object_begin(self): pass
    def object_end(self): pass
    def array_begin(self): pass
    def array_end(self): pass
    def string_begin(self): pass
    def string_end(self): pass
    def number_begin(self): pass
    def number_end(self): pass
    def char(self, c): pass
    def true(self): pass
    def false(self): pass
    def null(self): pass
    def fail(self, msg): pass

    def _assert(self, c, values):
        if c not in values:
            self.fail("%s not in %s" % (repr(c), repr(values)))
        return c

    def _read_space(self):
        while self.s.peek() in ' \t\r\n':
            self.s.next()

    def _read_pair(self):
        self.pair_begin()
        self._read_string()
        self._read_space()
        self._assert(self.s.next(), ':')
        self._read_space()
        self._read_value()
        self.pair_end()

    def _read_object(self):
        self.object_begin()
        self._assert(self.s.next(), '{')
        self._read_space()
        if self.s.peek() != '}':
            while True:
                self._read_pair()
                self._read_space()
                if self.s.peek() == '}':
                    break
                self._assert(self.s.next(), ',')
                self._read_space()
        self._assert(self.s.next(), '}')
        self.object_end()

    def _read_array(self):
        self.array_begin()
        self._assert(self.s.next(), '[')
        self._read_space()
        if self.s.peek() != ']':
            while True:
                self._read_value()
                self._read_space()
                if self.s.peek() == ']':
                    break
                self._assert(self.s.next(), ',')
                self._read_space()
        self._assert(self.s.next(), ']')
        self.array_end()

    def _read_char(self):
        c = self.s.next()
        if c == '\\':
            c = self.s.next()
            if c == 'b': c = '\b'
            elif c == 'f': c = '\f'
            elif c == 'n': c = '\n'
            elif c == 'r': c = '\r'
            elif c == 't': c = '\t'
            elif c == 'u':
                d1 = self.s.next()
                d2 = self.s.next()
                d3 = self.s.next()
                d4 = self.s.next()
                c = chr(int(d1+d2+d3+d4, 16))
            else: self._assert(c, '"\\/')
        self.char(c)

    def _read_string(self):
        self.string_begin()
        self._assert(self.s.next(), '"')
        while self.s.peek() != '"':
            self._read_char()
        self._assert(self.s.next(), '"')
        self.string_end()

    def _read_number(self):
        self.number_begin()
        # Note: We catch this here, as numbers don't have any special
        # ending character. If we end up with an EOF and ignore it
        # here, while the number is part of a bigger structure, the
        # parsing function for the surrounding structure will do
        # next/peek again and it will be re-raised.
        try:
            if self.s.peek() == '-':
                self.char(self.s.next())
            if self.s.peek() == '0':
                self.char(self.s.next())
            else:
                self._assert(self.s.peek(), '123456789')
                self.char(self.s.next())
                while self.s.peek() in '0123456789':
                    self.char(self.s.next())
            if self.s.peek() == '.':
                self.char(self.s.next())
                self._assert(self.s.peek(), '0123456789')
                while self.s.peek() in '0123456789':
                    self.char(self.s.next())
            if self.s.peek() in 'eE':
                self.char(self.s.next())
                if self.s.peek() in '+-':
                    self.char(self.s.next())
                self._assert(self.s.peek(), '0123456789')
                while self.s.peek() in '0123456789':
                    self.char(self.s.next())
        except EOFError:
            pass
        self.number_end()

    def _read_true(self):
        self._assert(self.s.next(), 't')
        self._assert(self.s.next(), 'r')
        self._assert(self.s.next(), 'u')
        self._assert(self.s.next(), 'e')
        self.true()

    def _read_false(self):
        self._assert(self.s.next(), 'f')
        self._assert(self.s.next(), 'a')
        self._assert(self.s.next(), 'l')
        self._assert(self.s.next(), 's')
        self._assert(self.s.next(), 'e')
        self.false()

    def _read_null(self):
        self._assert(self.s.next(), 'n')
        self._assert(self.s.next(), 'u')
        self._assert(self.s.next(), 'l')
        self._assert(self.s.next(), 'l')
        self.null()

    def _read_value(self):
        self._read_space()
        c = self.s.peek()
        if c == '{': return self._read_object()
        elif c == '[': return self._read_array()
        elif c == '"': return self._read_string()
        elif c == 't': return self._read_true()
        elif c == 'f': return self._read_false()
        elif c == 'n': return self._read_null()
        else: return self._read_number()

    def close(self):
        self.closable.close()

    def read_value(self):
        return self._read_value()

    def read_values(self):
        while True:
            self._read_value()


class Reader(Tokenizer):
    """A JSON parser that parses JSON strings read from a file-like
    object or character iterator (for example a string) into Python
    values.

    The parser must be instantiated with the file-like object or
    string as its sole argument. To actually parse any values, call
    either the read_value() method, or iterate over the return value
    of the read_values() method."""

    def __init__(self, s, object_initializer = None):
        Tokenizer.__init__(self, s)
        self.object_initializer = object_initializer
    def _struct_begin(self):
        self.state.append([])
    def _struct_end(self):
        self.state[-2].append(self.state[-1])
        del self.state[-1]
    def pair_begin(self): self._struct_begin()
    def pair_end(self): self._struct_end()
    def object_begin(self): self._struct_begin()
    def object_end(self):
        self.state[-1] = dict(self.state[-1])
        if '__jsonclass__' in self.state[-1]:
            self.class_object()
        self._struct_end()
    def class_object(self):
        if self.object_initializer and self.state[-1]['__jsonclass__'][0] in self.object_initializer:
            cls = self.state[-1].pop('__jsonclass__')
            params = cls[1:]
            cls = self.object_initializer[cls[0]]
            self.state[-1] = cls(params, self.state[-1])
    def array_begin(self): self._struct_begin()
    def array_end(self): self._struct_end()
    def string_begin(self): self.state.append(u"")
    def string_end(self):  self._struct_end()
    def number_begin(self): self.state.append(u"")
    def number_end(self):
        if '.' in self.state[-1] or 'e' in self.state[-1] or 'E' in self.state[-1]:
            self.state[-1] = float(self.state[-1])
        else:
            self.state[-1] = int(self.state[-1])
        self._struct_end()
    def char(self, c): self.state[-1] = self.state[-1] + c
    def true(self): self.state[-1].append(True)
    def false(self): self.state[-1].append(False)
    def null(self): self.state[-1].append(None)
    def fail(self, msg): raise Exception(msg)
    def read_value(self):
        self.state = [[]]
        self._read_value()
        return self.state[-1][-1]
    def read_values(self):
        try:
            while True:
                self.state = [[]]
                self._read_value()
                yield self.state[-1][-1]
        except EOFError:
            return


class DebugTokenizer:
    def pair_begin(self):
        logger.debug('(')
        logger.debug(self.state)
        return super(DebugTokenizer, self).pair_begin()

    def pair_end(self):
        logger.debug(')')
        logger.debug(self.state)
        return super(DebugTokenizer, self).pair_end()

    def object_begin(self):
        logger.debug('{')
        logger.debug(self.state)
        return super(DebugTokenizer, self).object_begin()

    def object_end(self):
        logger.debug('}')
        logger.debug(self.state)
        return super(DebugTokenizer, self).object_end()

    def array_begin(self):
        logger.debug('[')
        logger.debug(self.state)
        return super(DebugTokenizer, self).array_begin()

    def array_end(self):
        logger.debug(']')
        logger.debug(self.state)
        return super(DebugTokenizer, self).array_end()

    def string_begin(self):
        logger.debug('"')
        logger.debug(self.state)
        return super(DebugTokenizer, self).string_begin()

    def string_end(self):
        logger.debug('"')
        logger.debug(self.state)
        return super(DebugTokenizer, self).string_end()

    def number_begin(self):
        logger.debug('<')
        logger.debug(self.state)
        return super(DebugTokenizer, self).number_begin()

    def number_end(self):
        logger.debug('>')
        logger.debug(self.state)
        return super(DebugTokenizer, self).number_end()

    def char(self, c):
        logger.debug(repr(c))
        logger.debug(self.state)
        return super(DebugTokenizer, self).char(c)

    def true(self):
        logger.debug("TRUE")
        logger.debug(self.state)
        return super(DebugTokenizer, self).true()

    def false(self):
        logger.debug("FALSE")
        logger.debug(self.state)
        return super(DebugTokenizer, self).false()

    def null(self):
        logger.debug("NULL")
        logger.debug(self.state)
        return super(DebugTokenizer, self).null()

    def fail(self, msg):
        super(DebugTokenizer, self).fail()
        raise Exception(msg)


class DebugReader(DebugTokenizer, Reader):
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""JSON-RPC implementation classes."""

from __future__ import with_statement

import threading
from logging import getLogger

from . import dispatcher
from . import json


logger = getLogger(__name__)


class ClientConnection(dispatcher.Connection):
    """A connection manager for a connected socket (or similar) that
    reads and dispatches JSON values."""

    def _init(self, subject, parent=None, *arg, **kw):
        self.reader = json.Reader(subject)
        self.writer = json.Writer(subject)
        dispatcher.Connection._init(self, subject=subject, parent=parent, *arg, **kw)

    def shutdown(self):
        self.reader.close()
        self.writer.close()
        dispatcher.Connection.shutdown(self)

    def read(self):
        return self.reader.read_values()


class TimeoutError(Exception):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


class RPCClient(ClientConnection):
    """A JSON-RPC client connection manager.

    This class represents a single client-server connection on both
    the conecting and listening side. It provides methods for issuing
    requests and sending notifications, as well as handles incoming
    JSON-RPC request, responses and notifications and dispatches them
    in separate threads.

    The dispatched threads are instances of RPCClient.Dispatch, and
    you must subclass it and override the dispatch_* methods in it to
    handle incoming data."""

    class Request(dispatcher.ThreadedClient):
        def dispatch(self, subject):
            if 'method' in subject and 'id' in subject:
                try:
                    result = self.dispatch_request(subject)
                    error = None
                except Exception as e:
                    result = None
                    error = {'type': type(e).__name__,
                             'args': list(e.args)}
                self.parent.respond(result, error, subject['id'])
            elif 'result' in subject or 'error' in subject:
                assert 'id' in subject
                if subject['id'] in self.parent._recv_waiting:
                    with self.parent._recv_waiting[subject['id']]['condition']:
                        self.parent._recv_waiting[subject['id']]['result'] = subject
                        self.parent._recv_waiting[subject['id']]['condition'].notifyAll()
                else:
                    self.dispatch_response(subject)
            elif 'method' in subject:
                try:
                    self.dispatch_notification(subject)
                except:
                    logger.exception("dispatch_notification error")

        def dispatch_request(self, subject):
            pass

        def dispatch_notification(self, subject):
            pass

        def dispatch_response(self, subject):
            """Note: Only used to results for calls that some other thread isn't waiting for"""
            pass

    def _init(self, subject, parent=None, *arg, **kw):
        self._request_id = 0
        self._send_lock = threading.Lock()
        self._recv_waiting = {}
        ClientConnection._init(self, subject=subject, parent=parent, *arg, **kw)

    def request(self, method, params=[], wait_for_response=False, timeout=None):
        with self._send_lock:
            self._request_id += 1
            request_id = self._request_id
            if wait_for_response:
                self._recv_waiting[request_id] = {'condition': threading.Condition(), 'result': None}
            self.writer.write_value({'jsonrpc': '2.0', 'method': method, 'params': params, 'id': request_id})

            if not wait_for_response:
                return request_id

        try:
            with self._recv_waiting[request_id]['condition']:
                self._recv_waiting[request_id]['condition'].wait(timeout)
                if self._recv_waiting[request_id]['result'] == None:
                    raise TimeoutError("RPC timeout on method '{0}'".format(method))
                if self._recv_waiting[request_id]['result'].get('error') is not None:
                    exc = Exception(self._recv_waiting[request_id]['result']['error']['message'])
                    raise exc
                return self._recv_waiting[request_id]['result']['result']
        finally:
            del self._recv_waiting[request_id]

    def respond(self, result, error, id):
        with self._send_lock:
            self.writer.write_value({'result': result, 'error': error, 'id': id})

    def notify(self, method, params=[]):
        with self._send_lock:
            self.writer.write_value({'method': method, 'params': params})

    def __getattr__(self, name):
        def rpc_wrapper(*arg):
            return self.request(name, list(arg), wait_for_response=True)
        return rpc_wrapper


class RPCServer(dispatcher.ServerConnection):
    """A JSON-RPC server connection manager. This class manages a
    listening sockets and receives and dispatches new inbound
    connections. Each inbound connection is awarded two threads, one
    that can call the other side if there is a need, and one that
    handles incoming requests, responses and notifications.

    RPCServer.Dispatch.Dispatch is an RPCClient subclass that handles
    incoming requests, responses and notifications. Initial calls to
    the remote side can be done from its run_parent() method."""

    class InboundConnection(dispatcher.ThreadedClient):
        class Thread(RPCClient):
            def run_parent(self):
                """Server can call client from here..."""
                pass


class RPCP2PNode(dispatcher.ThreadedClient):
    class Thread(RPCServer):
        def run_parent(self):
            """Server can make connections from here by calling self.Dispatch()"""
            pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

import M2Crypto
import symmetricjsonrpc, socket

HOST='localhost'
PORT=4712

class PingRPCClient(symmetricjsonrpc.RPCClient):
    class Request(symmetricjsonrpc.RPCClient.Request):
        def dispatch_request(self, subject):
            # Handle callbacks from the server
            assert subject['method'] == "pingping"
            return "pingpong"

def main():
    ctx = M2Crypto.SSL.Context()
    ctx.set_verify(M2Crypto.SSL.verify_peer | M2Crypto.SSL.verify_fail_if_no_peer_cert, depth=9)
    if ctx.load_verify_locations('server.pem') != 1: raise Exception('No CA certs')
    s = M2Crypto.SSL.Connection(ctx)
    s.connect((HOST, PORT))
    client = PingRPCClient(s)
    res = client.request("ping", wait_for_response=False) == "pong"
    client.notify("shutdown")
    client.shutdown()

if __name__ == '__main__':
    main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import sys, time, threading

if '--help' in sys.argv:
    print("""client.py
    --ssl
        Encrypt communication with SSL using M2Crypto. Requires a
        server.pem in the current directory.
""")
    sys.exit(0)

if '--ssl' in sys.argv:
    # Set up an SSL connection
    import M2Crypto

    ctx = M2Crypto.SSL.Context()
    ctx.set_verify(M2Crypto.SSL.verify_peer | M2Crypto.SSL.verify_fail_if_no_peer_cert, depth=9)
    if ctx.load_verify_locations('server.pem') != 1: raise Exception('No CA certs')
    s = M2Crypto.SSL.Connection(ctx)
else:
    # Set up a TCP socket
    import socket
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

#  Connect to the server
s.connect(('localhost', 4712))

class ReaderThread(threading.Thread):
    def run(self, *arg, **kw):
        res = s.recv(1024)
        while res:
            print("rcv(%s)" % (repr(res),))
            res = s.recv(1024)

reader = ReaderThread()
reader.start()

s.send(b'{"params":[],"method":"ping","id":1}')
time.sleep(1)
s.send(b'{"params":[],"method":"ping","id":1}')



# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
import socket
import time
import unittest

from symmetricjsonrpc3 import json
from symmetricjsonrpc3.rpc import (ClientConnection, RPCClient, RPCP2PNode,
                                   RPCServer,
                                   dispatcher)


debug_tests = False


class _EchoDispatcher:
    def __init__(self, subject, parent):
        if not hasattr(parent, "writer"):
            parent = parent.parent
        parent.writer.write_value(subject)


class _EchoClient(ClientConnection):
    Request = _EchoDispatcher


class _ThreadedEchoClient(ClientConnection):
    class Request(dispatcher.ThreadedClient):
        Thread = _EchoDispatcher


class _EchoServer(dispatcher.ServerConnection):
    InboundConnection = _EchoClient


class _ThreadedEchoServer(dispatcher.ServerConnection):
    class InboundConnection(dispatcher.ThreadedClient):
        Thread = _ThreadedEchoClient


class _PingRPCClient(RPCClient):
    class Request(RPCClient.Request):
        def dispatch_request(self, subject):
            if debug_tests:
                print("PingClient: dispatch_request", subject)
            assert subject['method'] == "pingping"
            return "pingpong"


class _PongRPCServer(RPCServer):
    class InboundConnection(RPCServer.InboundConnection):
        class Thread(RPCServer.InboundConnection.Thread):
            class Request(RPCServer.InboundConnection.Thread.Request):
                def dispatch_request(self, subject):
                    if debug_tests:
                        print("TestPongRPCServer: dispatch_request", subject)
                    assert subject['method'] == "ping"
                    assert self.parent.request("pingping", wait_for_response=True) == "pingpong"
                    if debug_tests:
                        print("TestPongRPCServer: back-pong")
                    return "pong"


class _PongRPCP2PServer(RPCP2PNode):
    class Thread(RPCP2PNode.Thread):
        class InboundConnection(RPCP2PNode.Thread.InboundConnection):
            class Thread(RPCP2PNode.Thread.InboundConnection.Thread):
                class Request(RPCP2PNode.Thread.InboundConnection.Thread.Request):
                    def dispatch_request(self, subject):
                        if debug_tests:
                            print("TestPongRPCP2PServer: dispatch_request", subject)
                        if subject['method'] == "ping":
                            assert self.parent.request("pingping", wait_for_response=True) \
                                == "pingpong"
                            if debug_tests:
                                print("TestPongRPCServer: back-pong")
                            return "pong"
                        elif subject['method'] == "pingping":
                            if debug_tests:
                                print("PingClient: dispatch_request", subject)
                            return "pingpong"
                        else:
                            assert False

        def run_parent(self):
            client = self.InboundConnection.Thread(_make_client_socket())
            self.parent.parent['result'] = client.request("ping", wait_for_response=True) == "pong"


def _make_server_socket():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.bind(('', 4712))
    s.listen(1)
    return s


def _make_client_socket():
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(('localhost', 4712))
    return s


class TestRpc(unittest.TestCase):
    def test_client(self):
        sockets = socket.socketpair()
        _EchoClient(sockets[1])

        reader = json.Reader(sockets[0])
        writer = json.Writer(sockets[0])

        obj = {'foo': 1, 'bar': [1, 2]}
        writer.write_value(obj)
        return_obj = reader.read_value()
        self.assertEqual(obj, return_obj)

    def test_return_on_closed_socket(self):
        server_socket = _make_server_socket()
        echo_server = _EchoServer(server_socket, name="TestEchoServer")

        client_socket = _make_client_socket()
        writer = json.Writer(client_socket)
        writer.write_value({'foo': 1, 'bar': 2})
        client_socket.close()

        echo_server.shutdown()
        echo_server.join()

    def test_server(self):
        for n in range(3):
            server_socket = _make_server_socket()
            echo_server = _EchoServer(server_socket, name="TestEchoServer")

            client_socket = _make_client_socket()
            reader = json.Reader(client_socket)
            writer = json.Writer(client_socket)

            obj = {'foo': 1, 'bar': [1, 2]}
            writer.write_value(obj)
            return_obj = reader.read_value()

            self.assertEqual(obj, return_obj)
            echo_server.shutdown()
            echo_server.join()

    def test_threaded_server(self):
        for n in range(3):
            server_socket = _make_server_socket()
            echo_server = _ThreadedEchoServer(server_socket, name="TestEchoServer")

            client_socket = _make_client_socket()
            writer = json.Writer(client_socket)

            obj = {'foo': 1, 'bar': [1, 2]}
            writer.write_value(obj)

            reader = json.Reader(client_socket)
            return_obj = reader.read_value()

            self.assertEqual(obj, return_obj)
            echo_server.shutdown()
            echo_server.join()

    def test_rpc_server(self):
        for n in range(3):
            server_socket = _make_server_socket()
            server = _PongRPCServer(server_socket, name="PongServer")

            client_socket = _make_client_socket()
            client = _PingRPCClient(client_socket)
            self.assertEqual(client.request("ping", wait_for_response=True), "pong")
            self.assertEqual(client.ping(), "pong")
            server.shutdown()
            server.join()

    def test_rpc_p2p_server(self):
        for n in range(3):
            server_socket = _make_server_socket()
            res = {}
            server = _PongRPCP2PServer(server_socket, res, name="PongServer")
            for x in range(0, 4):
                if 'result' in res:
                    break
                time.sleep(1)
            assert 'result' in res and res['result']

            server.shutdown()
            server.join()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Facilities for managing a hierarchy of threads each providing a
synchronous I/O interface."""

import select
import threading
from logging import getLogger


logger = getLogger(__name__)


class Thread(threading.Thread):
    """This class is the base class for a set of threading.Thread
    subclasses that provides automatic start, some debugging print-out
    for start/stop, a subject resource to manage (like a socket), and
    possibly a child/parent relationship with another thread."""

    debug_thread = False

    def __init__(self, subject, parent=None, *arg, **kw):
        self._init(subject=subject, parent=parent, *arg, **kw)
        self.start()
        self.run_parent()

    def _init(self, subject, parent=None, *arg, **kw):
        self.children = []
        self.subject = subject
        self.parent = parent
        if hasattr(self.parent, "children"):
            self.parent.children.append(self)
        self._shutdown = False
        if 'name' not in kw:
            if self.parent:
                kw['name'] = "%s/%s" % (self.parent.name, type(self).__name__)
            else:
                kw['name'] = type(self).__name__
        threading.Thread.__init__(self, *arg, **kw)

    def _exit(self):
        for child in list(self.children):
            child.join()
        if hasattr(self.parent, "children"):
            self.parent.children.remove(self)

    def run(self, *arg, **kw):
        self._dbg("%s: BEGIN", self.name)
        self.run_thread(*arg, **kw)
        if self.debug_thread:  # perf check -- avoid the unnecessary join loop
            self._dbg("%s: TEARDOWN: %s", self.name,
                      ', '.join(child.name for child in self.children))
        self._exit()
        self._dbg("%s: END", self.name)

    def shutdown(self):
        self._dbg("%s: shutdown: %s",
                  threading.current_thread().name, self.name)
        for child in list(self.children):
            child.shutdown()
        self._shutdown = True
        self._dbg("%s: shutdown done: %s",
                  threading.current_thread().name, self.name)

    def run_parent(self):
        pass

    def run_thread(self, *arg, **kw):
        pass

    def _dbg(self, fmt, *args, **kwargs):
        if self.debug_thread:
            logger.debug(fmt, *args, **kwargs)


class Connection(Thread):
    """A connection manager thread base class."""

    debug_dispatch = False

    _dispatcher_class = "Request"

    def run_thread(self):
        for value in self.read():
            self._dbg("%s: DISPATCH: %s", self.name, value)
            self.dispatch(value)
            self._dbg("%s: DISPATCH DONE: %s", self.name, value)

    def read(self):
        pass

    def dispatch(self, subject):
        getattr(self, self._dispatcher_class)(parent=self, subject=subject)

    def _dbg(self, fmt, *args, **kwargs):
        if self.debug_dispatch:
            logger.debug(fmt, *args, **kwargs)


class ServerConnection(Connection):
    """Connection manager thread handling a listening socket,
    dispatching inbound connections."""

    _dispatcher_class = "InboundConnection"

    def read(self):
        poll = select.poll()
        poll.register(self.subject, select.POLLIN)

        while True:
            if self._shutdown:
                self.subject.close()
                return
            status = poll.poll(100)
            if status:
                socket, address = self.subject.accept()
                yield socket


class ThreadedClient(Thread):
    """A dispatch manager that can be used to wrap some other dispatch
    manager to have it started and entirely run inside an extra
    thread. This is actually useful to wrap connection managers with
    too, to have their run_parent() run inside a separate thread
    too. See the RPC module for a good example of this."""

    _dispatcher_class = "Thread"

    def _init(self, subject, parent=None, *arg, **kw):
        Thread._init(self, subject=subject, parent=parent, *arg, **kw)
        self.dispatch_subject = self.subject
        self.subject = getattr(self.parent, "subject", None)

    def run_thread(self):
        self.dispatch(self.dispatch_subject)

    def dispatch(self, subject):
        getattr(self, self._dispatcher_class)(parent=self, subject=subject)



# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
import unittest
from collections import OrderedDict

from symmetricjsonrpc3.json import Reader, Writer, from_json, to_json


class TestJson(unittest.TestCase):
    import socket
    import tempfile
    import threading

    def assertReadEqual(self, str, obj):
        reader = Reader(str)
        read_obj = reader.read_value()
        self.assertEqual(obj, read_obj)
        io = self.tempfile.TemporaryFile("w+")
        Writer(io).write_value(obj)
        io.seek(0)
        reader1 = Reader(io)
        read_obj1 = reader1.read_value()
        self.assertEqual(obj, read_obj1)

    def assertWriteEqual(self, str, obj):
        self.assertEqual(str, to_json(obj))

    def test_to_json(self):
        STR = '["string",false,null]'
        OBJ = ["string", False, None]
        self.assertEqual(to_json(OBJ), STR)

    def test_from_json(self):
        STR = '{"array": ["string",false,null],"object":{"number":4711,"bool":true}}'
        OBJ = {"array": ["string", False, None], "object": {"number": 4711, "bool": True}}
        self.assertEqual(from_json(STR), OBJ)

    def test_single_number_from_json(self):
        STR = '3.33'
        OBJ = 3.33
        self.assertEqual(from_json(STR), OBJ)

    def test_read_value(self):
        STR = '{"array": ["string",false,null],"object":{"number":4711,"bool":true}}'
        OBJ = {"array": ["string", False, None], "object": {"number": 4711, "bool": True}}
        self.assertReadEqual(STR, OBJ)

    def test_read_numbers(self):
        STR = '[0, -1, 0.2, 1e+4, -2.5E-5, 1e20]'
        self.assertReadEqual(STR, eval(STR))

    def test_read_escape_string(self):
        STR = r'"\b\f\n\r\t\u1234"'
        OBJ = "\b\f\n\r\t\u1234"
        self.assertReadEqual(STR, OBJ)

    def test_read_quote_string(self):
        STR = r'"\""'
        OBJ = "\""
        self.assertReadEqual(STR, OBJ)

    def test_read_solidus_string(self):
        STR = r'"\/"'
        OBJ = "/"
        self.assertReadEqual(STR, OBJ)

    def test_read_reverse_solidus_string(self):
        STR = r'"\\"'
        OBJ = "\\"
        self.assertReadEqual(STR, OBJ)

    def test_read_whitespace(self):
        STR = ''' {
"array" : [ ] ,
"object" : { }
} '''
        self.assertReadEqual(STR, eval(STR))

    def test_read_values(self):
        STR = "{}[]true false null"
        reader = Reader(STR)
        values = [{}, [], True, False, None]

        for i, r in enumerate(reader.read_values()):
            self.assertEqual(r, values[i])

    def test_encode_invalid_control_character(self):
        self.assertRaises(Exception, lambda: to_json('\x00'))

    def test_encode_invalid_object(self):
        self.assertRaises(Exception, lambda: to_json(object))

    def test_read_object(self):
        STR = '{"__jsonclass__":["foo","bar"],"naja":123}'

        def foo(arg, kw):
            assert arg == ["bar"]
            assert kw == {"naja": 123}
            return True
        reader = Reader(STR, {'foo': foo})
        assert reader.read_value() is True

    def test_broken_socket(self):
        sockets = self.socket.socketpair()
        reader = Reader(sockets[0])
        sockets[0].close()
        self.assertRaises(self.socket.error, lambda: reader.read_value())

    def test_eof(self):
        obj = {'foo': 1, 'bar': [1, 2]}
        io0 = self.tempfile.TemporaryFile("w+")
        Writer(io0).write_value(obj)
        io0.seek(0)
        full_json_string = io0.read()

        for json_string, eof_error in [(full_json_string, False),
                                       (full_json_string[0:10], True),
                                       ('', True)]:
            io1 = self.tempfile.TemporaryFile("w+")
            io1.write(json_string)
            io1.seek(0)
            reader = Reader(io1)
            if eof_error:
                self.assertRaises(EOFError, lambda: reader.read_value())
            else:
                self.assertEqual(obj, reader.read_value())

    def test_closed_socket(self):
        class Timeout(self.threading.Thread):
            def run(self1):
                obj = {'foo': 1, 'bar': [1, 2]}
                io = self.tempfile.TemporaryFile("w+")
                Writer(io).write_value(obj)
                io.seek(0)
                full_json_string = io.read()

                for json_string, eof_error in [(full_json_string, False),
                                               (full_json_string[0:10], True),
                                               ('', True)]:
                    sockets = self.socket.socketpair()
                    reader = Reader(sockets[0])

                    for c in json_string:
                        while not sockets[1].send(c.encode('ascii')):
                            pass
                    sockets[1].close()
                    if eof_error:
                        self.assertRaises(EOFError, lambda: reader.read_value())
                    else:
                        self.assertEqual(obj, reader.read_value())

        timeout = Timeout()
        timeout.start()
        timeout.join(3)
        if timeout.is_alive():
            self.fail('Reader has hung.')

    def test_write_object(self):
        class SomeObj:
            def __init__(self, x):
                self.x = x

            def __to_json__(self):
                return OrderedDict([('x', self.x), ('__jsonclass__', ['SomeObj'])])

        self.assertWriteEqual('{"x":4711,"__jsonclass__":["SomeObj"]}', SomeObj(4711))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# vim: set fileencoding=UTF-8 :

# python-symmetricjsonrpc3
# Copyright (C) 2009 Egil Moeller <redhog@redhog.org>
# Copyright (C) 2009 Nicklas Lindgren <nili@gulmohar.se>
# Copyright (C) 2024 Robert "Robikz" Zalewski <zalewapl@gmail.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""A symmetric and transport agnostic JSON-RPC implementation"""



from .json import *
from .dispatcher import *
from .rpc import *

__all__ = ["ClientConnection",
           "Connection",
           "RPCClient",
           "RPCP2PNode",
           "RPCServer",
           "Reader",
           "ServerConnection",
           "ShutDownThread",
           "Thread",
           "ThreadedClient",
           "Tokenizer",
           "Writer",
           "from_json",
           "to_json"]
